/*
 * Copyright 2004 Ronald Blaschke.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.rblasch.forrest.plugin;

import org.apache.avalon.framework.parameters.Parameters;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.environment.SourceResolver;
import org.apache.cocoon.generation.AbstractGenerator;
import org.apache.excalibur.source.Source;
import org.rblasch.jpod.PodToDocumentFormatter;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

import java.io.IOException;
import java.util.Map;

/**
 * @author Ronald Blaschke
 * @version 0.1
 * @cocoon.sitemap.component.documentation XXX
 * @cocoon.sitemap.component.name pod
 * @cocoon.sitemap.component.label content
 * @cocoon.sitemap.component.logger sitemap.generator.imagedirectory
 * @cocoon.sitemap.component.documentation.caching Uses the last modification date of the directory and the contained files
 * @cocoon.sitemap.component.pooling.min 2
 * @cocoon.sitemap.component.pooling.max 16
 * @cocoon.sitemap.component.pooling.grow 2
 */
public class PodGenerator extends AbstractGenerator {
    AttributesImpl emptyAttr = new AttributesImpl();

    private Source source;

    public void setup(final SourceResolver resolver,
                      final Map objectModel,
                      final String src,
                      final Parameters parameters)
            throws ProcessingException, SAXException, IOException {
        super.setup(resolver, objectModel, src, parameters);
        this.source = resolver.resolveURI(src);
    }

    /**
     * Override the generate() method from AbstractGenerator.
     * It simply generates SAX events using SAX methods.
     * I haven't done the comparison myself, but this
     * has to be faster than parsing them from a string.
     */

    public void generate() throws IOException, SAXException, ProcessingException {
        // the org.xml.sax.ContentHandler is inherited
        // through org.apache.cocoon.xml.AbstractXMLProducer
        try {
            try {
                final PodToDocumentFormatter formatter = new PodToDocumentFormatter(source.getInputStream(), contentHandler, lexicalHandler, null);
                formatter.generate();
            } catch (final Exception ex) {
                throw new ProcessingException(ex);
            }
        } catch (final Exception ex) {
            ex.printStackTrace(System.err);
        }
    }
}
